Morgan Stanley makes this available to you under the Apache License, Version 2.0 (the "License"). You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0. See the NOTICE file distributed with this work for additional information regarding copyright ownership.
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the specific language governing permissions and limitations under the License.

# Contributing

When contributing to this repository, please first discuss the change you wish 
to make via issue, email, or any other method with the owners of this repository
before making a change.

Please note we have a code of conduct, please follow it in all your interactions
with the project.

## Before your first pull request

If you have not previously contributed to the project, 
you must first create a *Developer Certificate of Origin* (“DCO”) and include a 
reference to this DCO in each of your commits.  In addition, if you subsequently 
wish to contribute code having a different copyright ownership, then you must create 
a new DCO for such contribution.

To create a DCO please follow these steps:

1. For code you are contributing, determine who is/are the copyright owner(s). 
Please note that your employer may own the copyright in code you have written even 
where the code was not created during regular working hours.  Copyright law is
variable from jurisdiction to jurisdiction. Accordingly, consult your employer
or a lawyer if you are not sure.
2. Fill out the [DCO](dco/dco_template.md) replacing all `<>` terms as appropriate, and place the 
completed DCO in a file under `dco/<your name>` or if you are not the copyright 
holder then in a file under `dco/<your name>-<copyright holder name(s)>`.
    1. Please note that the name you provide (`<your name>`) must be your real 
       (legal) name; we will not accept aliases, pseudonyms or anonymous 
       contributions.
    1. If you’ve determined that the copyright holder of the code that you’ve 
       written is an entity other than yourself (e.g., your employer), then 
       include the legal name of the copyright holder(s) (`<name of copyright holder(s)>`).
       You must ensure that you are authorized by the copyright holder(s) to be able 
       to grant the licenses under the DCO for the purpose of contributing to the 
       project. Negotiating such authorization and administering the terms is 
       entirely between you and the copyright holder(s).
3. Issue a pull request with the DCO.

## Pull request process 

When you create a pull request, follow these steps:

1. Your commit message for the code you are submitting must include a
`“Covered by <dco>“` line which indicates your acceptance of the DCO terms and conditions. 
`<dco>` here is the file name of the DCO.
2. Your commit must include a change to the `NOTICE` file that contains complete
details of any applicable copyright notice for your submission and including any
applicable third party license(s) or other restrictions associated with any part
of your contribution, and of all matters required to be disclosed under such third
party license(s) (such as any applicable copyright, patent, trademark, and attribution
notices, and any notices relating to modifications made to open source software).
Note your contribution must retain all applicable copyright, patent, trademark and
attribution notices.

## Pull request guidelines

* Update the README.md/docs with details of changes to the interface.
* Update an existing or add a new testcase for your change.
* Ensure any install or build artifacts are removed from the pull request.
* We generally prefer squashed commits, unless multi-commits add clarity or are required for mixed copyright commits.
* You may merge the Pull Request in once the build has passed and you have the
   sign-off of one other developer, or if you do not have permission to do that,
   you may request the reviewer to merge it for you.

## Code of Conduct

### Our Pledge

In the interest of fostering an open and welcoming environment, we as
contributors and maintainers pledge to making participation in our project and
our community a harassment-free experience for everyone, regardless of age, body
size, disability, ethnicity, gender identity and expression, level of experience,
nationality, personal appearance, race, religion, or sexual identity and
orientation.

### Our Standards

Examples of behavior that contributes to creating a positive environment
include:

* Using welcoming and inclusive language
* Being respectful of differing viewpoints and experiences
* Gracefully accepting constructive criticism
* Focusing on what is best for the community
* Showing empathy towards other community members

Examples of unacceptable behavior by participants include:

* The use of sexualized language or imagery and unwelcome sexual attention or
advances
* Trolling, insulting/derogatory comments, and personal or political attacks
* Public or private harassment
* Publishing others' private information, such as a physical or electronic
  address, without explicit permission
* Other conduct which could reasonably be considered inappropriate in a
  professional setting

### Our Responsibilities

Project maintainers are responsible for clarifying the standards of acceptable
behavior and are expected to take appropriate and fair corrective action in
response to any instances of unacceptable behavior.

Project maintainers have the right and responsibility to remove, edit, or
reject comments, commits, code, wiki edits, issues, and other contributions
that are not aligned to this Code of Conduct, or to ban temporarily or
permanently any contributor for other behaviors that they deem inappropriate,
threatening, offensive, or harmful.

### Scope

This Code of Conduct applies both within project spaces and in public spaces
when an individual is representing the project or its community. Examples of
representing a project or community include using an official project e-mail
address, posting via an official social media account, or acting as an appointed
representative at an online or offline event. Representation of a project may be
further defined and clarified by project maintainers.

### Enforcement

Instances of abusive, harassing, or otherwise unacceptable behavior may be
reported by contacting the project team at desktopjs [at] morganstanley.com. All
complaints will be reviewed and investigated and will result in a response that
is deemed necessary and appropriate to the circumstances. The project team is
obligated to maintain confidentiality with regard to the reporter of an incident.
Further details of specific enforcement policies may be posted separately.

Project maintainers who do not follow or enforce the Code of Conduct in good
faith may face temporary or permanent repercussions as determined by other
members of the project's leadership.

### Attribution

This Code of Conduct is adapted from the [Contributor Covenant][homepage],
version 1.4, available at [http://contributor-covenant.org/version/1/4][version]

[homepage]: http://contributor-covenant.org
[version]: http://contributor-covenant.org/version/1/4/
